@@ -8,9 +8,11 @@
 #' @param res Resolution of the faces.
 #' @return A list with named elements "obj", "mtl", "png" with the created filenames.
 #' @examples
-#'     cfg <- game_systems("sans3d")$dominoes
-#'     files <- save_piece_obj("tile_face", suit = 3+1, rank=6+1, cfg = cfg)
-#'     print(files)
+#'     if (all(capabilities(c("cairo", "png")))) {
+#'       cfg <- game_systems("sans3d")$dominoes
+#'       files <- save_piece_obj("tile_face", suit = 3+1, rank=6+1, cfg = cfg)
+#'       print(files)
+#'     }
 #' @seealso See \code{\link{geometry_utils}} for a discussion of the 3D rotation parameterization.
 #' @export
 save_piece_obj <- function(piece_side = "tile_face", suit = 1, rank = 1,
@@ -20,6 +22,9 @@ save_piece_obj <- function(piece_side = "tile_face", suit = 1, rank = 1,
                            angle = 0, axis_x = 0, axis_y = 0,
                            width = NA, height = NA, depth = NA,
                            filename = tempfile(fileext = ".obj"), scale = 1, res = 72) {
+    opt <- options(piecepackr.op_scale = 0)
+    on.exit(options(opt))
+
     cfg <- as_pp_cfg(cfg)
     suit <- ifelse(is.na(suit), 1, suit)
     rank <- ifelse(is.na(rank), 1, rank)
@@ -54,8 +59,11 @@ write_2s_texture <- function(piece_side = "tile_face", suit = 1, rank = 1, cfg =
     width <- cfg$get_width(piece_face, suit, rank)
     edge_color <- cfg$get_piece_opt(piece_face, suit, rank)$edge_color
 
-    grDevices::png(filename, height = height, width = 2.5 * width,
-        units = "in", res = res, bg = "transparent")
+    args <- list(filename = filename, height = height, width = 2.5 * width,
+                 units = "in", res = res, bg = "transparent")
+    if (capabilities("cairo"))
+        args$type <- "cairo"
+    do.call(grDevices::png, args)
 
     # front
     pushViewport(viewport(x = 0.225, width = 0.45))
@@ -378,13 +386,16 @@ write_peg_doll_texture <- function(piece_side = "pawn_face", suit = 1, rank = 1,
     width <- cfg$get_width("belt_face", suit, rank)
     piece <- get_piece(piece_side)
     opt <- cfg$get_piece_opt(paste0(piece, "_face"), suit, rank)
-    grDevices::png(filename, height = 2 * height, width = width,
-        units = "in", res = res, bg = "transparent")
+    args <- list(filename = filename, height = 2 * height, width = width,
+                 units = "in", res = res, bg = "transparent")
+    if (capabilities("cairo"))
+        args$type <- "cairo"
+    do.call(grDevices::png, args)
     pushViewport(viewport(y=0.875, height=0.25))
     grid.rect(gp = gpar(col=NA_character_, fill=opt$edge_color))
     popViewport()
     pushViewport(viewport(y=0.500, height=0.50))
-    grid.piece("belt_face", suit = suit, rank = rank, cfg = cfg)
+    grid.piece("belt_face", suit = suit, rank = rank, cfg = cfg, op_scale = 0)
     popViewport()
     pushViewport(viewport(y=0.125, height=0.25))
     grid.rect(gp = gpar(col=NA_character_, fill=opt$edge_color))
@@ -403,25 +414,10 @@ save_die_obj <- function(piece_side = "die_face", suit = 1, rank = 1, cfg = pp_c
 
     cfg <- as_pp_cfg(cfg)
     opt <- cfg$get_piece_opt(piece_side, suit, rank)
-    pc <- Point3D$new(x, y, z)
-
-    xs <- c(0, 0, 1, 1, 0, 0, 1, 1) - 0.5
-    ys <- c(1, 0, 0, 1, 1, 0, 0, 1) - 0.5
-    zs <- rep(c(1, 0), each = 4) - 0.5
-
-    # figure out rotation to reach each die face
-    rs <- get_die_face_info(suit, cfg$die_arrangement) #### also allow customization of angle #175
-    i <- which(rs$rank == rank)
-    dR <- switch(i,
-                 diag(3),
-                 R_y(-90) %*% R_z(90),
-                 R_x(90) %*% R_z(-90),
-                 R_x(180),
-                 R_y(90) %*% R_z(90),
-                 R_x(-90) %*% R_z(90),
-                 diag(3))
-    R <- dR %*% AA_to_R(angle, axis_x, axis_y)
-    xyz <- Point3D$new(xs, ys, zs)$dilate(width, height, depth)$rotate(R)$translate(pc)
+    xyz <- die_xyz(suit, rank, cfg,
+                   x, y, z,
+                   angle, axis_x, axis_y,
+                   width, height, depth)
 
     xy_vt <- list(x = rep(c(0, 0.5, 1), 4),
                   y = rep(c(1, 2/3, 1/3, 0), each = 3))
@@ -454,8 +450,11 @@ write_pyramid_texture <- function(piece_side = "pyramid_face", suit = 1, rank =
     height <- cfg$get_height("pyramid_face", suit, rank)
     width <- cfg$get_width("pyramid_face", suit, rank)
 
-    grDevices::png(filename, height = height, width = 4 * width,
-        units = "in", res = res, bg = "transparent")
+    args <- list(filename = filename, height = height, width = 4 * width,
+                 units = "in", res = res, bg = "transparent")
+    if (capabilities("cairo"))
+        args$type <- "cairo"
+    do.call(grDevices::png, args)
 
     pushViewport(viewport(x = 0.125, width = 0.25))
     draw_piece_and_bleed("pyramid_face", suit, rank, cfg)
@@ -484,27 +483,36 @@ write_die_texture <- function(piece_side = "die_face", suit = 1, rank = 1, cfg =
     if (current_dev > 1) on.exit(grDevices::dev.set(current_dev))
     width <- cfg$get_width("die_face", suit, rank)
 
-    grDevices::png(filename, height = 3 * width, width = 2 * width,
-                   units = "in", res = res, bg = "transparent")
+    args <- list(filename = filename, height = 3 * width, width = 2 * width,
+                 units = "in", res = res, bg = "transparent")
+    if (capabilities("cairo"))
+        args$type <- "cairo"
+    do.call(grDevices::png, args)
 
-    rs <- get_die_face_info(suit, cfg$die_arrangement) #### also angle
+    rs <- get_die_face_info(suit, cfg$die_arrangement)
     pushViewport(viewport(x = 0.25, width = 0.5, y = 5/6, height = 1/3))
-    grid.piece("die_face", suit = rs$suit[1], rank = rs$rank[1], cfg = cfg)
+    grid.piece("die_face", suit = rs$suit[1], rank = rs$rank[1], cfg = cfg,
+               angle = rs$angle[1], op_scale = 0)
     popViewport()
     pushViewport(viewport(x = 0.75, width = 0.5, y = 5/6, height = 1/3))
-    grid.piece("die_face", suit = rs$suit[2], rank = rs$rank[2], cfg = cfg)
+    grid.piece("die_face", suit = rs$suit[2], rank = rs$rank[2], cfg = cfg,
+               angle = rs$angle[2], op_scale = 0)
     popViewport()
     pushViewport(viewport(x = 0.25, width = 0.5, y = 3/6, height = 1/3))
-    grid.piece("die_face", suit = rs$suit[3], rank = rs$rank[3], cfg = cfg)
+    grid.piece("die_face", suit = rs$suit[3], rank = rs$rank[3], cfg = cfg,
+               angle = rs$angle[3], op_scale = 0)
     popViewport()
     pushViewport(viewport(x = 0.75, width = 0.5, y = 3/6, height = 1/3))
-    grid.piece("die_face", suit = rs$suit[4], rank = rs$rank[4], cfg = cfg)
+    grid.piece("die_face", suit = rs$suit[4], rank = rs$rank[4], cfg = cfg,
+               angle = rs$angle[4], op_scale = 0)
     popViewport()
     pushViewport(viewport(x = 0.25, width = 0.5, y = 1/6, height = 1/3))
-    grid.piece("die_face", suit = rs$suit[5], rank = rs$rank[5], cfg = cfg)
+    grid.piece("die_face", suit = rs$suit[5], rank = rs$rank[5], cfg = cfg,
+               angle = rs$angle[5], op_scale = 0)
     popViewport()
     pushViewport(viewport(x = 0.75, width = 0.5, y = 1/6, height = 1/3))
-    grid.piece("die_face", suit = rs$suit[6], rank = rs$rank[6], cfg = cfg)
+    grid.piece("die_face", suit = rs$suit[6], rank = rs$rank[6], cfg = cfg,
+               angle = rs$angle[6], op_scale = 0)
     popViewport()
 
     grDevices::dev.off()
@@ -512,7 +520,7 @@ write_die_texture <- function(piece_side = "die_face", suit = 1, rank = 1, cfg =
 }
 
 draw_piece_and_bleed <- function(piece_side, suit, rank, cfg) {
-    g <- pieceGrob(piece_side, suit, rank, cfg, default.units = "npc")
+    g <- pieceGrob(piece_side, suit, rank, cfg, default.units = "npc", op_scale = 0)
     bleed_color <- cfg$get_piece_opt(piece_side, suit, rank)$bleed_color
     b <- pp_shape()$polyclip(g, "minus", gp=gpar(col=NA, fill=bleed_color))
     grid.draw(b)
@@ -530,15 +538,9 @@ save_pt_obj <- function(piece_side = "pyramid_top", suit = 1, rank = 1, cfg = pp
     cfg <- as_pp_cfg(cfg)
 
     # vertices
-    pc <- Point3D$new(x, y, z)
-    xy_npc <- Point2D$new(rect_xy)
-    xy <- xy_npc$translate(-0.5, -0.5)
-    xyz_t <- Point3D$new(x = 0.0, y = 0.0, z = 0.5)
-    xyz_b <- Point3D$new(xy, z = -0.5)
-    xs <- c(xyz_t$x, xyz_b$x)
-    ys <- c(xyz_t$y, xyz_b$y)
-    zs <- c(xyz_t$z, xyz_b$z)
-    xyz <- Point3D$new(xs, ys, zs)$dilate(width, height, depth)$rotate(angle, axis_x, axis_y)$translate(pc)
+    xyz <- pt_xyz(x, y, z,
+                  angle, axis_x, axis_y,
+                  width, height, depth)
 
     # texture coordinates
     xy_vt <- list(x = seq(0, 1, 0.125), y = rep(c(0, 1), length.out = 9))
@@ -571,16 +573,9 @@ save_ps_obj <- function(piece_side = "pyramid_face", suit = 1, rank = 1, cfg = p
     cfg <- as_pp_cfg(cfg)
 
     # geometric vertices
-    pc <- Point3D$new(x, y, z)
-    xy_npc <- Point2D$new(pyramid_xy)
-    xy <- xy_npc$translate(-0.5, -0.5)
-    xyz_b <- Point3D$new(xy, z = -0.5)
-    theta <- 2 * asin(0.5 * width / height)
-    xyz_t <- Point3D$new(x = c(-0.5, 0.5), y = 0.5 - cos(theta), z = 0.5)
-    xs <- c(xyz_t$x, xyz_b$x)
-    ys <- c(xyz_t$y, xyz_b$y)
-    zs <- c(xyz_t$z, xyz_b$z)
-    xyz <- Point3D$new(xs, ys, zs)$dilate(width, height, depth)$rotate(angle, axis_x, axis_y)$translate(pc)
+    xyz <- ps_xyz(x, y, z,
+                  angle, axis_x, axis_y,
+                  width, height, depth)
 
     # texture coordinates
     xy_vt <- list(x = seq(0, 1, 0.125), y = rep(c(0, 1), length.out = 9))
